#!/usr/bin/env ruby

##
# Ruby API for Flickr (See http://www.flickr.com/services/api/)
# (C) 2005 Premshree Pillai
# http://www.livejournal.com/~premshree
##

# $Id: flickr.rb,v 1.6 2006/06/12 13:05:48 premshree Exp $

require 'cgi'
require 'md5'
require 'net/http'
require 'rexml/document'
include REXML

module Flickr

class Flickr

	attr_reader :nsid, :username, :fullname

	@@secret, @@api_key, @@interface = '', '', 'http://www.flickr.com/services/rest/'

	def initialize(api_key, secret)
		@auth_url = 'http://flickr.com/services/auth/'
		@@secret = secret
		@@api_key = api_key
		@nsid = ''
		@username = ''
		@fullname = ''
	end

	def get_frob
		method = 'flickr.auth.getFrob'
		api_sig = _get_api_sig({'method' => method})
		data = _do_get(method, {
					'api_sig' => api_sig
				}
			)
		doc = Document.new(data)
		doc.elements.each('rsp/frob') { |frob|
			return frob.text
		}
	end

	##
	# perms:
	#  'read', 'write', 'delete'
	##
	def get_login_url(frob=nil, perms='read')
		if frob != nil	
			api_sig = _get_api_sig({'frob' => frob, 'perms' => perms});
		else
			api_sig = _get_api_sig({'perms' => perms})
		end
		ret = "#{@auth_url}?api_key=#{@@api_key}&perms=#{perms}&api_sig=#{api_sig}"
		ret = ret + "&frob=#{frob}" if frob != nil
		return ret
	end

	def get_token(frob)
		method = 'flickr.auth.getToken'
		api_sig = _get_api_sig({'frob' => frob, 'method' => method})
		data = _do_get(method, {
					'api_sig'	=> api_sig,
					'frob'		=> frob
				}
			)
		doc = Document.new(data)
		doc.elements.each('rsp/auth') { |auth|
			token = auth.elements['token'].text
			user = auth.elements['user']
			@nsid = user.attributes['nsid']
			@username = user.attributes['username']
			@fullname = user.attributes['fullname']
			return token
		}
	end

	def check_token(auth_token)
		method = 'flickr.auth.checkToken'
                api_sig = _get_api_sig(
            				{
            				'method'	=> method,
            				'auth_token'	=> auth_token
            			}
            		)
                data = _do_get(method, {
                                        'api_sig' => api_sig,
                                        'auth_token' => auth_token
                                } 
                        )
		return false if !data 
                doc = Document.new(data)
                doc.elements.each('rsp/auth') { |auth|
			perms = auth.elements['perms'].text
			user = auth.elements['user']
			@nsid = user.attributes['nsid']
                        @username = user.attributes['username']
                        @fullname = user.attributes['fullname']
			return true        
                }
		return false 
	end
  
	def _do_get(method, *params)
		params.each { |x, y|
			if y.class == Array then params[x] = y.join(y) end
		}
		url = "#{@@interface}?api_key=#{@@api_key}&method=#{method}#{_urlencode(params)}"
		resp = Net::HTTP.get_response(URI.parse(url))
		doc = Document.new(resp.body.to_s)
		if doc.elements['rsp'].attributes['stat'] != 'ok'
			error_code = doc.elements['rsp'].elements['err'].attributes['code']
			error_msg = doc.elements['rsp'].elements['err'].attributes['msg']
			msg = "ERROR[#{error_code}]: #{error_msg}"
			p msg
			return false	
		end
		resp.body.to_s
	end
	def _do_get2(method, *params)
		params.each { |x, y|
			if y.class == Array then params[x] = y.join(y) end
		}
		url = "#{@@interface}?api_key=#{@@api_key}&method=#{method}#{_urlencode(params)}"
		TextMate.exit_insert_text url
	end

	def _urlencode(params)
		ret = ''
		params.each { |param|
			param.each { |x, y|
				ret += "&#{x}=#{y}"
			}
		}
		ret
	end

	def _get_api_sig(params)
		ret = ''
		params.sort.each { |x, y|
			ret += "#{x}#{y}"
		}
		MD5.md5("#{@@secret}api_key#{@@api_key}#{ret}")
	end

end

class Contacts < Flickr

	def initialize
	end

	##
	# filter:
	#  'friends', 'family', 'both', 'neither'
	##
	def get_list(auth_token, filter='')
		method = 'flickr.contacts.getList'
		api_sig = _get_api_sig(
				{
				'method'	=> method,
				'auth_token'	=> auth_token,
				'filter'	=> filter
			}
		)
		data = _do_get(method,
					{
					'api_sig'	=> api_sig,
					'auth_token'	=> auth_token,
					'filter'	=> filter
				}
			)
		doc = Document.new(data)
		contacts = []
		doc.elements.each('rsp/contacts/contact') { |contact|
			contacts << {
				'nsid'		=> contact.attributes['nsid'],
				'username'	=> contact.attributes['username'],
				'realname'	=> contact.attributes['realname'],
				'friend'	=> contact.attributes['friend'],
				'family'	=> contact.attributes['family'],
				'ignored'	=> contact.attributes['ignored']
				}
		}
		return contacts
	end

	def get_public_list(nsid)
		method = 'flickr.contacts.getPublicList'
		data = _do_get(method,
					{
					'api_key'	=> @@api_key,
					'user_id'	=> nsid
				}
			)
		doc = Document.new(data)
		contacts = []
		doc.elements.each('rsp/contacts/contact') { |contact|
			contacts << {
				'nsid'		=> contact.attributes['nsid'],
				'username'	=> contact.attributes['username'],
				'ignored'	=> contact.attributes['ignored']
			}
		}
		return contacts
	end
end

class Favorites < Flickr

	def initialize
	end

	def add(auth_token, photo_id)
		method = 'flickr.favorites.add'
		api_sig = _get_api_sig(
				{
				'method'	=> method,
				'auth_token'	=> auth_token,
				'photo_id'	=> photo_id
			}
		)
		data = _do_get(method, {
					'api_sig'	=> api_sig,
					'auth_token'	=> auth_token,
					'photo_id'	=> photo_id
				}
			)
		if data
			return true
		end
	end

	##
	# extras: array that consists one or more of
	# license, date_upload, date_taken, owner_name, icon_server
	##
	def get_list(auth_token, nsid='', extras=[], per_page=100, page=1)
		method = 'flickr.favorites.getList'
		api_sig = _get_api_sig(
				{
				'method'	=> method,
				'auth_token'	=> auth_token,
				'user_id'	=> nsid,
				'extras'	=> extras.join(','),
				'per_page'	=> per_page,
				'page'		=> page
			}
		)
		data = _do_get(method,
				{
				'api_sig'	=> api_sig,
				'auth_token'	=> auth_token,
				'user_id'	=> nsid,
				'extras'	=> extras.join(','),
				'per_page'	=> per_page,
				'page'		=> page
			}
		)
		doc = Document.new(data)
		photos = []
		doc.elements.each('rsp/photos/photo') { |photo|
			photos << {
				'id'		=> photo.attributes['id'],
				'owner'		=> photo.attributes['owner'],
				'ispublic'	=> photo.attributes['ispublic'],
				'isfamily'	=> photo.attributes['isfamily'],
				'isfriend'	=> photo.attributes['isfriend'],
				'title'		=> photo.attributes['title'],
				'license'	=> photo.attributes['license'],
				'owner_name'	=> photo.attributes['ownername'],
				'date_taken'	=> photo.attributes['datetaken'],
				'date_upload'	=> photo.attributes['dateupload']
			}
		}
		return photos
	end

	##
	# extras: array that consists one or more of
	# license, date_upload, date_taken, owner_name, icon_server
	##
	def get_public_list(nsid, extras=[], per_page=100, page=1)
		method = 'flickr.favorites.getPublicList'
		data = _do_get(method,
				{
				'api_key'	=> @@api_key,
				'user_id'	=> nsid,
				'extras'	=> extras.join(','),
				'per_page'	=> per_page,
				'page'		=> page
			}
		)
		doc = Document.new(data)
		photos = []
		doc.elements.each('rsp/photos/photo') { |photo|
			photos << {
				'id'		=> photo.attributes['id'],
				'owner'		=> photo.attributes['owner'],
				'ispublic'	=> photo.attributes['ispublic'],
				'isfamily'	=> photo.attributes['isfamily'],
				'isfriend'	=> photo.attributes['isfriend'],
				'title'		=> photo.attributes['title'],
				'license'	=> photo.attributes['license'],
				'owner_name'	=> photo.attributes['ownername'],
				'date_taken'	=> photo.attributes['datetaken'],
				'date_upload'	=> photo.attributes['dateupload']
			}
		}
		return photos
	end

	def remove(auth_token, photo_id)
		method = 'flickr.favorites.remove'
		api_sig = _get_api_sig(
				{
				'method'	=> method,
				'auth_token'	=> auth_token,
				'photo_id'	=> photo_id
			}
		)
		data = _do_get(method, {
					'api_sig'	=> api_sig,
					'auth_token'	=> auth_token,
					'photo_id'	=> photo_id
				}
			)
		if data
			return true
		end
	end
end

class Photos < Flickr

	def initialize
	end

	def add_tags(auth_token, photo_id, tags)
		method = 'flickr.photos.addTags'
		api_sig = _get_api_sig(
				{
				'method'	=> method,
				'auth_token'	=> auth_token,
				'photo_id'	=> photo_id,
				'tags'		=> tags.join(' ')
			}
		)
		data = _do_get(method, {
					'api_sig'	=> api_sig,
					'auth_token'	=> auth_token,
					'photo_id'	=> photo_id,
					'tags'		=> tags.join(' ')
				}
			)
		if data
			return true
		end
	end

	##
	# max(count) = 50
	##
	def get_contacts_photos(auth_token, count=10, just_friends=0, single_photo=0, include_self=0)
		method = 'flickr.photos.getContactsPhotos'
		api_sig = _get_api_sig(
				{
				'method'	=> method,
				'auth_token'	=> auth_token,
				'count'		=> count,
				'just_friends'	=> just_friends,
				'single_photo'	=> single_photo,
				'include_self'	=> include_self
			}
		)
		data = _do_get(method, {
					'api_sig'	=> api_sig,
					'auth_token'	=> auth_token,
					'count'		=> count,
					'just_friends'	=> just_friends,
					'single_photo'	=> single_photo,
					'include_self'	=> include_self
				}
		)
		doc = Document.new(data)
		photos = []
		doc.elements.each('rsp/photos/photo') { |photo|
			photos << {
				'id'		=> photo.attributes['id'],
				'owner'		=> photo.attributes['owner'],
				'username'	=> photo.attributes['username'],
				'title'		=> photo.attributes['title']
			}
		}
		return photos
	end
	def get_photo_sets(auth_token)
		method = 'flickr.photosets.getList'
		api_sig = _get_api_sig(
				{
				'method'	=> method,
				'auth_token'	=> auth_token,
				}
		)
		data = _do_get(method, {
					'api_sig'	=> api_sig,
					'auth_token'	=> auth_token,
				}
		)
		doc = Document.new(data)
		photosets = []
		doc.elements.each('rsp/photosets/photoset') { |photoset|
			photosets << {
				'id'		=> photoset.attributes['id'],
				'primary'		=> photoset.attributes['primary'],
				'secret'	=> photoset.attributes['secret'],
				'server'		=> photoset.attributes['server'],
				'farm'  => photoset.attributes['farm'],
				'count' => photoset.attributes['photos'],
				'title'		=> photoset.elements['title'].text,
				'description'		=> photoset.elements['description'].text
			}
		}
		return photosets
	end
  def get_photos(photoset)
		method = 'flickr.photosets.getPhotos'
		data = _do_get(method, {
					'api_key'	=> @@api_key,
					'photoset_id'		=> photoset
				})
		doc = Document.new(data)
		photos = []
		doc.elements.each('rsp/photoset/photo') { |photo|
			photos << {
				'id'		=> photo.attributes['id'],
				'title'		=> photo.attributes['title'],
				'secret'	=> photo.attributes['secret'],
				'server'		=> photo.attributes['server']
			}
		}
		return photos
	end
	##
	# max(count) = 50
	##
	def get_contacts_public_photos(nsid, count=10, just_friends=0, single_photo=0, include_self=0)
		method = 'flickr.photos.getContactsPublicPhotos'
		data = _do_get(method, {
					'api_key'	=> @@api_key,
					'user_id'	=> nsid,
					'count'		=> count,
					'just_friends'	=> just_friends,
					'single_photo'	=> single_photo,
					'include_self'	=> include_self
				}
		)
		doc = Document.new(data)
		photos = []
		doc.elements.each('rsp/photos/photo') { |photo|
			photos << {
				'id'		=> photo.attributes['id'],
				'owner'		=> photo.attributes['owner'],
				'username'	=> photo.attributes['username'],
				'title'		=> photo.attributes['title']
			}
		}
		return photos
	end

	def get_info(photo_id)
		method = 'flickr.photos.getInfo'
		data = _do_get(method, {
					'api_key'	=> @@api_key,
					'photo_id'	=> photo_id
				}
		)
		doc = Document.new(data)
		doc.elements.each('rsp/photo') { |photo|
			owner = photo.elements['owner']
			date = photo.elements['dates']
			tags = photo.elements['tags']
			tags_arr = []
			photo.elements.each('tags/tag') { |tag|
				tags_arr << tag.text
			}
			urls = photo.elements['urls']
			urls_arr = []
			photo.elements.each('urls/url') { |url|
				urls_arr << url.text
			}
			return {
				'id'		=> photo.attributes['id'],
				'isfavorite'	=> photo.attributes['isfavorite'],
				'license'	=> photo.attributes['license'],
				'title'		=> photo.elements['title'].text,
				'owner_nsid'	=> owner.attributes['nsid'],
				'owner_username'=> owner.attributes['username'],
				'owner_realname'=> owner.attributes['realname'],
				'owner_location'=> owner.attributes['location'],
				'date_posted'	=> date.attributes['posted'],
				'date_taken'	=> date.attributes['taken'],
				'tags'		=> tags_arr,
				'urls'		=> urls_arr
			}
		}
	end
	
	def get_sizes(photo_id)
		method = 'flickr.photos.getSizes'
		data = _do_get(method, {
					'api_key'	=> @@api_key,
					'photo_id'	=> photo_id
				}
		)
		sizes = []
		doc = Document.new(data)
		doc.elements.each('rsp/sizes/size') { |size|
			sizes << {
				'label'		=> size.attributes['label'],
				'width'		=> size.attributes['width'],
				'height'	=> size.attributes['height'],
				'source'		=> size.attributes['source']
			}
			}
			return sizes
	end

	# url_type is one of {url, source}
	def get_sizes_old(photo_id, size='Medium', url_type='source')
                method = 'flickr.photos.getSizes'
                data = _do_get(method, {
					'api_key'	=> @@api_key,
					'photo_id'	=> photo_id,
					'size'		=> size
				})
                doc = Document.new(data)
                doc.elements.each('rsp/sizes/size') { |x|
                        if x.attributes['label'] == size
				return x.attributes[url_type]
                                if url_type == 'url'
                                        return x.attributes['url']
                                end
                                if url_type == 'source'
                                        return x.attributes['source']
                                end
                        end
                }
	end

	##
	# TODO: more args 
	##
	def search_old(tags, tag_mode='any', license=5, per_page=100, sort="interestingness-desc")
		method = 'flickr.photos.search'
		data = _do_get(method, {
					'api_key'	=> @@api_key,
					'tags'		=> tags.join(","),
					'tag_mode'	=> tag_mode,
					'license'	=> license,
					'per_page'	=> per_page,
					'sort'		=> sort
				})
		doc = Document.new(data)
		photo_ids = []
		doc.elements.each('rsp/photos/photo') { |x|
			photo_ids << x.attributes['id']
		}
		photo_ids	
	end

	def search(text,nsid=@nsid)
		method = 'flickr.photos.search'
		data = _do_get(method, {
					'api_key'	=> @@api_key,
					'text'	=> text,
					'user_id'	=> nsid
				})
		doc = Document.new(data)
		photos = []
		doc.elements.each('rsp/photos/photo') { |photo|
			photos << {
				'id'		=> photo.attributes['id'],
				'secret'		=> photo.attributes['secret'],
				'server'	=> photo.attributes['server'],
				'title'		=> photo.attributes['title']
			}
			}
			return photos
	end
end


class Upload < Flickr

	def initialize
	end

	def _to_multipart(name, value)
		return "Content-Disposition: form-data; name=\"#{CGI::escape(name)}\"\r\n\r\n#{value}\r\n"
	end
	
	def _file_to_multipart(name, file, content)
		return "Content-Disposition: form-data; name=\"#{CGI::escape(name)}\"; filename=\"#{file}\"\r\n" +
			  "Content-Transfer-Encoding: binary\r\n" +
			  "Content-Type: image/jpeg\r\n\r\n" + content + "\r\n"
	end
	
	def _prepare_query(params)
		query = params.collect { |k, v|
			if v.respond_to?(:read)
				q = _file_to_multipart(k, v.path, v.read)
			else
				q = _to_multipart(k, v)
			end
			"--" + @boundary + "\r\n" + q 
		}.join("") + "--" + @boundary + "--"
		header = {"Content-type" => "multipart/form-data, boundary=" + @boundary + " "}
		return query, header
	end

	def upload(auth_token, photo, title='', description='', tags=[], is_public=1,\
		   is_friend=0, is_family=0)
		@flickr_host = 'www.flickr.com'
		@upload_action = '/services/upload/'
		@boundary = MD5.md5(photo).to_s
		file = File.new(photo, 'rb')
		api_sig = _get_api_sig(
				{
				'auth_token'	=> auth_token,
				'title'		=> title,
				'description'	=> description,
				'tags'		=> tags.join(' '),
				'is_public'	=> is_public,
				'is_family'	=> is_family,
				'is_friend'	=> is_friend
			}
		)
		params = {
			'api_key'	=> @@api_key,
			'api_sig'	=> api_sig,
			'auth_token'	=> auth_token,
			'photo'		=> file,
			'title'		=> title,
			'description'	=> description,
			'tags'		=> tags.join(' '),
			'is_public'	=> is_public,
			'is_family'	=> is_family,
			'is_friend'	=> is_friend
		}
		query, header = _prepare_query(params)
		file.close
		Net::HTTP.start(@flickr_host, 80) { |http|
			response = http.post(@upload_action, query, header)
			doc = Document.new(response.body)
			status = doc.elements['rsp'].attributes['stat']
			if status == "ok"
				photoid = doc.elements['rsp/photoid'].text
				return photoid
			else
				return false
			end
		}
	end
end

class People < Flickr

	def initialize
	end

        def get_public_photos(user_id, per_page='', page='')
                method = 'flickr.people.getPublicPhotos'
                data = _do_get(method, {
					'user_id'=>user_id,
                       			'per_page'=>per_page,
					'page'=>page
					})
                p = []
                doc = Document.new(data)
                doc.elements.each('rsp/photos/photo') { |photo|
                        id = photo.attributes['id']
                        owner = photo.attributes['owner']
                        title = photo.attributes['title']
                        ispublic = photo.attributes['ispublic']
                        isfriend = photo.attributes['isfriend']
                        isfamily = photo.attributes['isfamily']
			photo_obj = {
				'id'		=> id, 
				'title'		=> title,
				'ispublic'	=> ispublic,
				'isfriend'	=> isfriend,
				'isfamily'	=> isfamily
			}
                        p << photo_obj
                }
                p
        end

end

end
